@@ -34,26 +34,27 @@ Full example::
       })
       Schema: ClassVar[Type[Schema]] = Schema # For the type checker
 """
+import dataclasses
 import inspect
+import warnings
 from enum import EnumMeta
 from functools import lru_cache
 from typing import (
-    overload,
-    Dict,
-    Type,
-    List,
-    cast,
-    Tuple,
-    Optional,
     Any,
+    Callable,
+    Dict,
+    List,
     Mapping,
+    Optional,
+    Set,
+    Tuple,
+    Type,
     TypeVar,
     Union,
-    Callable,
-    Set,
+    cast,
+    overload,
 )
 
-import dataclasses
 import marshmallow
 import typing_inspect
 
@@ -69,6 +70,33 @@ MEMBERS_WHITELIST: Set[str] = {"Meta"}
 MAX_CLASS_SCHEMA_CACHE_SIZE = 1024
 
 
+@overload
+def dataclass(
+    _cls: Type[_U],
+    *,
+    repr: bool = True,
+    eq: bool = True,
+    order: bool = False,
+    unsafe_hash: bool = False,
+    frozen: bool = False,
+    base_schema: Optional[Type[marshmallow.Schema]] = None,
+) -> Type[_U]:
+    ...
+
+
+@overload
+def dataclass(
+    *,
+    repr: bool = True,
+    eq: bool = True,
+    order: bool = False,
+    unsafe_hash: bool = False,
+    frozen: bool = False,
+    base_schema: Optional[Type[marshmallow.Schema]] = None,
+) -> Callable[[Type[_U]], Type[_U]]:
+    ...
+
+
 # _cls should never be specified by keyword, so start it with an
 # underscore.  The presence of _cls is used to detect if this
 # decorator is being called with parameters or not.
@@ -138,7 +166,7 @@ def add_schema(_cls=None, base_schema=None):
     This decorator adds a marshmallow schema as the 'Schema' attribute in a dataclass.
     It uses :func:`class_schema` internally.
 
-    :param type cls: The dataclass to which a Schema should be added
+    :param type _cls: The dataclass to which a Schema should be added
     :param base_schema: marshmallow schema used as a base class when deriving dataclass schema
 
     >>> class BaseSchema(marshmallow.Schema):
@@ -155,6 +183,7 @@ def add_schema(_cls=None, base_schema=None):
     """
 
     def decorator(clazz: Type[_U]) -> Type[_U]:
+        # noinspection PyTypeHints
         clazz.Schema = class_schema(clazz, base_schema)  # type: ignore
         return clazz
 
@@ -173,7 +202,7 @@ def class_schema(
     :return: A marshmallow Schema corresponding to the dataclass
 
     .. note::
-        All the arguments supported by marshmallow field classes are can
+        All the arguments supported by marshmallow field classes can
         be passed in the `metadata` dictionary of a field.
 
 
@@ -258,12 +287,6 @@ def class_schema(
         ...
     marshmallow.exceptions.ValidationError: {'_schema': ['never valid']}
 
-    >>> # noinspection PyTypeChecker
-    >>> class_schema(None) # unsupported type
-    Traceback (most recent call last):
-      ...
-    TypeError: None is not a dataclass and cannot be turned into one.
-
     >>> @dataclasses.dataclass
     ... class Anything:
     ...     name: str
@@ -275,20 +298,30 @@ def class_schema(
     ...
     marshmallow.exceptions.ValidationError: {'name': ['Name too long']}
     """
-    return _proxied_class_schema(clazz, base_schema)
+    if not dataclasses.is_dataclass(clazz):
+        clazz = dataclasses.dataclass(clazz)
+    return _internal_class_schema(clazz, base_schema)
 
 
 @lru_cache(maxsize=MAX_CLASS_SCHEMA_CACHE_SIZE)
-def _proxied_class_schema(
+def _internal_class_schema(
     clazz: type, base_schema: Optional[Type[marshmallow.Schema]] = None
 ) -> Type[marshmallow.Schema]:
-
     try:
         # noinspection PyDataclass
         fields: Tuple[dataclasses.Field, ...] = dataclasses.fields(clazz)
     except TypeError:  # Not a dataclass
         try:
-            return class_schema(dataclasses.dataclass(clazz), base_schema)
+            warnings.warn(
+                f"marshmallow_dataclass was called on the class {clazz}, which is not a dataclass. "
+                f"It is going to try and convert the class into a dataclass, which may have "
+                f"undesirable side effects. To avoid this message, make sure all your classes and "
+                f"all the classes of their fields are either explicitly supported by "
+                f"marshmallow_datcalass, or are already dataclasses. For more information, see "
+                f"https://github.com/lovasoa/marshmallow_dataclass/issues/51"
+            )
+            created_dataclass: type = dataclasses.dataclass(clazz)
+            return _internal_class_schema(created_dataclass, base_schema)
         except Exception:
             raise TypeError(
                 f"{getattr(clazz, '__name__', repr(clazz))} is not a dataclass and cannot be turned into one."
@@ -324,6 +357,44 @@ def _field_by_type(
     ) or marshmallow.Schema.TYPE_MAPPING.get(typ)
 
 
+def _field_by_supertype(
+    typ: Type,
+    default: marshmallow.missing,
+    newtype_supertype: Type,
+    metadata: dict,
+    base_schema: Optional[Type[marshmallow.Schema]],
+) -> marshmallow.fields.Field:
+    """
+    Return a new field for fields based on a super field. (Usually spawned from NewType)
+    """
+    # Add the information coming our custom NewType implementation
+
+    typ_args = getattr(typ, "_marshmallow_args", {})
+
+    # Handle multiple validators from both `typ` and `metadata`.
+    # See https://github.com/lovasoa/marshmallow_dataclass/issues/91
+    new_validators: List[Callable] = []
+    for meta_dict in (typ_args, metadata):
+        if "validate" in meta_dict:
+            if marshmallow.utils.is_iterable_but_not_string(meta_dict["validate"]):
+                new_validators.extend(meta_dict["validate"])
+            elif callable(meta_dict["validate"]):
+                new_validators.append(meta_dict["validate"])
+    metadata["validate"] = new_validators if new_validators else None
+
+    metadata = {"description": typ.__name__, **typ_args, **metadata}
+    field = getattr(typ, "_marshmallow_field", None)
+    if field:
+        return field(**metadata)
+    else:
+        return field_for_schema(
+            newtype_supertype,
+            metadata=metadata,
+            default=default,
+            base_schema=base_schema,
+        )
+
+
 def field_for_schema(
     typ: type,
     default=marshmallow.missing,
@@ -404,41 +475,41 @@ def field_for_schema(
                 values=field_for_schema(arguments[1], base_schema=base_schema),
                 **metadata,
             )
-        elif typing_inspect.is_optional_type(typ):
-            subtyp = next(t for t in arguments if t is not NoneType)  # type: ignore
-            # Treat optional types as types with a None default
-            metadata["default"] = metadata.get("default", None)
-            metadata["missing"] = metadata.get("missing", None)
-            metadata["required"] = False
-            return field_for_schema(subtyp, metadata=metadata, base_schema=base_schema)
         elif typing_inspect.is_union_type(typ):
-            subfields = [
-                field_for_schema(subtyp, metadata=metadata, base_schema=base_schema)
-                for subtyp in arguments
-            ]
-            import marshmallow_union
+            if typing_inspect.is_optional_type(typ):
+                metadata["default"] = metadata.get("default", None)
+                metadata["missing"] = metadata.get("missing", None)
+                metadata["required"] = False
+            subtypes = [t for t in arguments if t is not NoneType]  # type: ignore
+            if len(subtypes) == 1:
+                return field_for_schema(
+                    subtypes[0], metadata=metadata, base_schema=base_schema
+                )
+            from . import union_field
 
-            return marshmallow_union.Union(subfields, **metadata)
+            return union_field.Union(
+                [
+                    (
+                        subtyp,
+                        field_for_schema(
+                            subtyp, metadata=metadata, base_schema=base_schema
+                        ),
+                    )
+                    for subtyp in subtypes
+                ],
+                **metadata,
+            )
 
     # typing.NewType returns a function with a __supertype__ attribute
     newtype_supertype = getattr(typ, "__supertype__", None)
     if newtype_supertype and inspect.isfunction(typ):
-        # Add the information coming our custom NewType implementation
-        metadata = {
-            "description": typ.__name__,
-            **getattr(typ, "_marshmallow_args", {}),
-            **metadata,
-        }
-        field = getattr(typ, "_marshmallow_field", None)
-        if field:
-            return field(**metadata)
-        else:
-            return field_for_schema(
-                newtype_supertype,
-                metadata=metadata,
-                default=default,
-                base_schema=base_schema,
-            )
+        return _field_by_supertype(
+            typ=typ,
+            default=default,
+            newtype_supertype=newtype_supertype,
+            metadata=metadata,
+            base_schema=base_schema,
+        )
 
     # enumerations
     if isinstance(typ, EnumMeta):
@@ -452,7 +523,7 @@ def field_for_schema(
     # Nested dataclasses
     forward_reference = getattr(typ, "__forward_arg__", None)
     nested = (
-        nested_schema or forward_reference or class_schema(typ, base_schema=base_schema)
+        nested_schema or forward_reference or _internal_class_schema(typ, base_schema)
     )
 
     return marshmallow.fields.Nested(nested, **metadata)
@@ -535,8 +606,11 @@ def NewType(
         return x
 
     new_type.__name__ = name
+    # noinspection PyTypeHints
     new_type.__supertype__ = typ  # type: ignore
+    # noinspection PyTypeHints
     new_type._marshmallow_field = field  # type: ignore
+    # noinspection PyTypeHints
     new_type._marshmallow_args = kwargs  # type: ignore
     return new_type
 
